<?php

namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use CRM\Geral\Controllers;
use Teknisa\Model\Util\NovoCodigo;

class crm30000000CompromissoController extends TeknisaCrud
{
    protected $dataSourceName = "planocupacao_Data_Hora";
    
    protected $planoOcupacaoService;
    protected $enviarMensagemController;
    protected $environment;
    
    public function __construct($crudService, $parameterBag, $planoOcupacaoService, $enviarMensagemController, $environment, $parametrizacaoService) {
        
        $this->environment = $environment;
        $this->planoOcupacaoService = $planoOcupacaoService;
        $this->enviarMensagemController = $enviarMensagemController;
        $this->parametrizacaoService = $parametrizacaoService;
        
        parent::__construct($crudService, $parameterBag);
    }
    
    public function getContatoCompromisso(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'contatoCompromisso';
        $this->parameterBagColumns = array();
        parent::find($request, $response);
    }
    
    public function save(Request $request, Response $response) {
        
        $retorno = array();
        $cdoperadorLogado = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrgTrab();
            
        foreach ($request->getDataSet()->getRows() as &$row) {
            if($row['__is_new']){
                $row['NRSEQPLANOCUPA'] = NovoCodigo::geraCodigo($nrorg, "PLANOCUPACAO");
                $row['NRSEQPLANOCUPA'] = (int)$row['NRSEQPLANOCUPA'];
                
                if($row['CDOPERADORESPONS'] != $cdoperadorLogado){
                    $params = $this->parametrizacaoService->callGetCrmParametrization(array('NRORG' => $nrorg))[0];
                    if(empty($params['DSEMAILSMTP']) || empty($params['DSHOSTSMTP']) || empty($params['DSSENHASMTP']) || empty($params['DSPORTASMTP'])) {
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("error", array("Preencha os dados de envio de e-mail na tela Parametrização CRM!")));
                    }else {
                        $this->notificarResponsavelCompromisso($row);
                    }
                }
            }
            $retorno['NRSEQPLANOCUPA'] = $row['NRSEQPLANOCUPA'];
            $row['CDFILIAL'] = $this->environment->getCdFilial();
            $row['HRINIPLANOCUPA'] = isset($row['HRINIPLANOCUPA']) ? $row['HRINIPLANOCUPA'] : substr($row['DTINIPLANOCUPA'], 11, 5); 
            $row['HRFINPLANOCUPA'] = isset($row['HRFINPLANOCUPA']) ? $row['HRFINPLANOCUPA'] : substr($row['DTFINPLANOCUPA'], 11, 5); 
        }
        //Para evitar o erro "Cannot find row to update!"
        $this->dataSourceName = "planocupacao_Data_Hora_consulta";
        
        parent::save($request, $response);
        $this->dataSourceName = "planocupacao_Data_Hora";
        $response->addDataSet(new DataSet("columnsToReturn", $retorno));
        
    }
    
    public function insertCommitmetnPart(Row $row){
        $row = $row->getRow();
        for($i = 0; $i < count($row['CDCOMMITMENTPARTS']); $i++){
            
            $this->planoOcupacaoService->executeInsertCommitmentPart($row, $i);
        
        }
    }
    
    public function getCommitmentNrseqplanocupa(Row $row, Response $response){
        $row = $row->getRow();
        
        $nrseqplanocupa = $this->planoOcupacaoService->executeSelectNrseqplanocupa($row);
        $response->addDataset(new DataSet('DADOS', $nrseqplanocupa));
    }
    
    public function getCommitmentParts(Row $row, Response $response){
        $row = $row->getRow();
        
        $commitmentParts = $this->planoOcupacaoService->executeSelectCommitmentParts($row);
        $response->addDataset(new DataSet('DADOS', $commitmentParts));
    }
    
    public function updateStatusPlanocupacao(Row $row){
        $row = $row->getRow();
        
        $this->planoOcupacaoService->executeUpdateStatusPlanoca($row);
        
    }
    
    public function deleteCommitmentParts(Row $row){
        $row = $row->getRow();
        
        $this->planoOcupacaoService->executeDeleteCommitmentParts($row);
        
    }
    
    public function deleteCommitment(Row $row){
        $row = $row->getRow();
        
        $this->planoOcupacaoService->executeDeleteCommitment($row);
        
    }
    
    public function notificarCompromisso(DTO\Request\Row $row, DTO\Response $response){
        $message = null;
        $returnDsEmailOper = null;
        $returnAmbos       = null;
        $dataSet           = $this->montaNovaRow($row->getRow());
        $nrorg             = $this->environment->getNrOrgTrab();
        $params = $this->parametrizacaoService->callGetCrmParametrization(array('NRORG' => $nrorg))[0];
        if(empty($params['DSEMAILSMTP']) || empty($params['DSHOSTSMTP']) || empty($params['DSSENHASMTP']) || empty($params['DSPORTASMTP'])) {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("error", array("Preencha os dados de envio de e-mail na tela Parametrização CRM!")));
            return;
        }
        $commitmentParts   = $this->planoOcupacaoService->executeSelectCommitmentParts($dataSet);
        $commitmentParts   = array_merge($commitmentParts, $this->planoOcupacaoService->getInterlocs($nrorg, $dataSet));

        if(is_array($commitmentParts)){
            $cont = 0;
            foreach($commitmentParts as $Part){
                if(!empty($Part['TIPO']) && strtoupper($Part['TIPO']) === "INTERLOCUTOR") {
                    $nrseqinterlocutor = $Part['NRSEQINTERLOCUTOR'];
                    $cdoperador = null;
                }else {
                    $cdoperador = $Part['CDOPERADOR'];
                    $nrseqinterlocutor = null;
                }
                
                $from = $this->environment->getCdOperador();
                $return = $this->enviarMensagemController->enviarEmail($dataSet['MENSAGEM'], $nrorg, $cdoperador, $from, null, null, null, null, $nrseqinterlocutor);
                if($return == 'DSEMAILOPER'){
                    if(!is_null($returnDsEmailOper)){
                        $returnDsEmailOper .= ' ';
                    }
                    $returnDsEmailOper.= !empty($Part['NMOPERADOR']) ? $Part['NMOPERADOR'] : $Part['NMINTERLOCUTOR'];
                } else if ($return == 'AMBOS'){
                    if(!is_null($returnAmbos)){
                        $returnAmbos .= ' ';
                    }
                    $returnAmbos .= !empty($Part['NMOPERADOR']) ? $Part['NMOPERADOR'] : $Part['NMINTERLOCUTOR'];
                }
                $cont++;
            }
            if(!is_null($returnDsEmailOper)){
                if(!is_null($message)){
                    $message .= '<br><br>';
                }
                $message .= 'O email do(s) participante(s) a seguir não foram encontrados: ' + $returnDsEmailOper;
            }

            if(!is_null($returnAmbos)){
                if(!is_null($message)){
                    $message .= '<br><br>';
                }
                $message .= 'O(s) participante(s) a seguir não foram encontrados: ' + $returnAmbos;
            }
        
            if ($cont != 0) {
                $response->addMessage(new Message("$cont e-mail(s) enviado(s) com sucesso!", Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new Message("Nenhum email foi enviado.", Message::TYPE_WARNING));
            }
        
            if(!is_null($message)){
                $response->addNotification(new Notification($message, Notification::TYPE_SUCCESS));
            }
        }
    }
    
    // Monta uma row com os interlocutores passados na widget e os tira da parte da mensagem
    public function montaNovaRow($row) {
        if(isset($row['MENSAGEM']['TIPO'])) {
            foreach($row['MENSAGEM']['TIPO'] as $tipo) {
                if(!isset($row['TIPO']) || !is_array($row['TIPO'])) {
                    $row['TIPO'] = array($tipo);
                } else {
                    array_push($row['TIPO'], $tipo);
                }
            }
        }
        
        if(isset($row['MENSAGEM']['CDOPERADORPART'])) {
            foreach($row['MENSAGEM']['CDOPERADORPART'] as $cdoperadorpart) {
                if(!isset($row['CDOPERADORPART']) || !is_array($row['CDOPERADORPART'])) {
                    $row['CDOPERADORPART'] = array($cdoperadorpart);
                } else {
                    array_push($row['CDOPERADORPART'], $cdoperadorpart);
                }
            }
        }
        
        if(isset($row['MENSAGEM']['DSEMAILOPER'])) {
            foreach($row['MENSAGEM']['DSEMAILOPER'] as $dsemailoper) {
                if(!isset($row['DSEMAILOPER']) || !is_array($row['DSEMAILOPER'])) {
                    $row['DSEMAILOPER'] = array($dsemailoper);
                } else {
                    array_push($row['DSEMAILOPER'], $dsemailoper);
                }
            }
        }
        
        if(isset($row['MENSAGEM']['NMOPERADORPART'])) {
            foreach($row['MENSAGEM']['NMOPERADORPART'] as $nmoperadorpart) {
                if(!isset($row['NMOPERADORPART']) || !is_array($row['NMOPERADORPART'])) {
                    $row['NMOPERADORPART'] = array($nmoperadorpart);
                } else {
                    array_push($row['NMOPERADORPART'], $nmoperadorpart);
                }
            }
        }
        
        if(isset($row['MENSAGEM']['CDOPERADOR'])) {
            foreach($row['MENSAGEM']['CDOPERADOR'] as $nrseqinterloc) {
                if(!isset($row['NRSEQINTERLOC']) || !is_array($row['NRSEQINTERLOC'])) {
                    $row['NRSEQINTERLOC'] = array($nrseqinterloc);
                } else {
                    array_push($row['NRSEQINTERLOC'], $nrseqinterloc);
                }
            }
        }
        
        if(isset($row['MENSAGEM']['NRPARCNEGOCIO'])) {
            foreach($row['MENSAGEM']['NRPARCNEGOCIO'] as $nrparcnegocio) {
                if(!isset($row['NRPARCNEGOCIO']) || !is_array($row['NRPARCNEGOCIO'])) {
                    $row['NRPARCNEGOCIO'] = array($nrparcnegocio);
                } else {
                    array_push($row['NRPARCNEGOCIO'], $nrparcnegocio);
                }
            }
        }
        
        if(isset($row['MENSAGEM']['NRORGOPERADORPART'])) {
            foreach($row['MENSAGEM']['NRORGOPERADORPART'] as $nrorgoperadorpart) {
                if(!isset($row['NRORGOPERADORPART']) || !is_array($row['NRORGOPERADORPART'])) {
                    $row['NRORGOPERADORPART'] = array($nrorgoperadorpart);
                } else {
                    array_push($row['NRORGOPERADORPART'], $nrorgoperadorpart);
                }
            }
        }
        
        if(isset($row['NRSEQPLANOCUPA']) && is_array($row['NRSEQPLANOCUPA'])) {
            $row['NRSEQPLANOCUPA'] = $row['NRSEQPLANOCUPA'][0];
        }
        
        
        unset($row['MENSAGEM']['TIPO']);
        unset($row['MENSAGEM']['CDOPERADORPART']);
        unset($row['MENSAGEM']['DSEMAILOPER']);
        unset($row['MENSAGEM']['NMOPERADORPART']);
        unset($row['MENSAGEM']['CDOPERADOR']);
        unset($row['MENSAGEM']['NRPARCNEGOCIO']);
        unset($row['MENSAGEM']['NRORGOPERADORPART']);
        
        return $row;
    }
    
    public function notificarResponsavelCompromisso($row) {
        $params = $this->parametrizacaoService->callGetCrmParametrization(array('NRORG' => $row['NRORG']))[0];
        if(empty($params['DSEMAILSMTP']) || empty($params['DSHOSTSMTP']) || empty($params['DSSENHASMTP']) || empty($params['DSPORTASMTP'])) {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("error", array("Preencha os dados de envio de e-mail na tela Parametrização CRM!")));
            return;
        }
        $from = $this->environment->getCdOperador();
        $nmoperador = $this->planoOcupacaoService->getOperador($from, $row['NRORG'])[0];

        $mensagem = array();
        $mensagem['DSMENSAGEM']       = '<table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee" style="background-color: #eeeeee;"> <tbody> <tr> <td> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="0274BA" class="container590" style="background-color: #0274ba; text-align: center;"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="center" valign="middle"> <table border="0" align="middle" cellpadding="0" cellspacing="0" style="border-collapse:collapse; mso-table-lspace:0pt; mso-table-rspace:0pt;" class=""> <tbody> <tr> <td align="center" valign="middle"> <p style="color: #ffffff; font-size: 25px; padding:20px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 25px; padding-top:45px;">NOTIFICAÇÃO DE COMPROMISSO </p> <p style="color: #ffffff; font-size: 16px; font-family: "Questrial", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"> [DATACOMPROMISSO] </p> <p style="color: #ffffff; font-size: 16px; font-family: "Questrial", Helvetica, sans-serif; line-height: 15px; padding-top:15px;">&nbsp;</p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"> <br> </p> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> <table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee"> <tbody> <tr> <td align="center"> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="ffffff" class="container590 bg_color"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="left" style="color: #39434b; font-size: 20px; font-family: " Montserrat ", Helvetica, sans-serif; font-weight: 400; padding-bottom:15px; padding-top:20px; line-height: 35px;" class="title_color"> <div style="padding:20px; line-height: 22px"> Prezado(a) [PARTICIPANTE]</div> </td> </tr> <tr> <td align="left" class="title_color" style="color: #848999; font-size: 14px; font-family: " Questrial ", sans-serif; mso-line-height-rule: exactly; line-height: 24px; padding-bottom:15px;"> <div style="line-height: 24px;"> <p>O Operador [OPERADOR] inseriu você como operador responsável no compromisso referente a(ao) [TIPO] [REFERENCIA]. <br> Segue os dados do Compromisso: . <ul> <li> <p>Parceiro: [PARCEIRO]</p> </li> <li> <p>Data de Início: [DATACOMPROMISSO] </p> </li> <li> <p>Data de Fim: [DATAFINAL] </p> </li> <li> <p>Motivo: [MOTIVO] </p> </li> </ul> </p> <br> </div> </td> </tr> <tr> <td height="10" style="font-size: 10px; line-height: 10px;">&nbsp;</td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table>';
        $mensagem['DSTITULOMENSAGEM'] = 'CRM - Notificação de Criação de Compromisso';
        $mensagem['PARTICIPANTE']     = ucwords(strtolower($row['CDOPERADORESPONS']));   
        $mensagem['OPERADOR']         = ucwords(strtolower($nmoperador['NMOPERADOR']));
        $mensagem['PARCEIRO']         = ucwords(strtolower($row['NMPRINCIPALPARC']));
        $mensagem['DATACOMPROMISSO']  = $row['DTINI'];
        $mensagem['DATAFINAL']        = $row['DTFIM'];
        $mensagem['MOTIVO']           = $row['NMMOTIEVOCON'];
        
        $return = $this->enviarMensagemController->enviarEmail($mensagem, $row['NRORG'], $row['CDOPERADORESPONS'], $from, null, null, null, null, null);
    }
    
}