
// FILE: src/config/config.js
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-image-uploader';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}

// FILE: src/controllers/ImageUploaderController.js
var ImageUploaderController = function($scope, templateManager, ScreenService) {

	var self = this;

	var widgetAccessorDescriptor = {
		get: function() {
			return $scope.widget;
		}
	};

	var fieldAcessosDescriptor = {
		get: function() {
			return $scope.field;
		}
	};

	Object.defineProperty(self, "widget", widgetAccessorDescriptor);
	Object.defineProperty(self, "field", fieldAcessosDescriptor);

	/***************************************************
	 * Functions used by template
	 ***************************************************/
	this.getChooseFileText = function() {
		return self.field.chooseFileText || ScreenService.i18n("Choose a file...");
	};

	this.getNoFileSelectedText = function() {
		var currentFieldRow = self.widget.currentRow[self.field.name];
		if (currentFieldRow && (typeof(currentFieldRow) === 'object')) {
			self.field.noFileSelectedText = currentFieldRow[0].name;
		} else if (currentFieldRow) {
			self.field.noFileSelectedText = currentFieldRow;
		} else {
			self.field.noFileSelectedText = ScreenService.i18n("No file selected.");
		}
		return self.field.noFileSelectedText || ScreenService.i18n("No file selected.");
	};

	this.getCurrentThumbnail = function() {
		var currentFieldRow = self.widget.currentRow[self.field.name];
		var currentThumbnail = "//:0";
		if (typeof currentFieldRow === "string") {
			var root = templateManager.project.fileServiceBaseUrl || "";
			currentThumbnail = root + currentFieldRow;
		} else if (Array.isArray(currentFieldRow) && currentFieldRow.length > 0) {
			currentThumbnail = currentFieldRow[0].b64File;
		}

		return currentThumbnail;
	};

	this.getCurrentThumbnailStringify = function(){
		return JSON.stringify(self.getCurrentThumbnail());
	};

	this.isVisibleInCurrentFormTemplate = function() {};

	this.isEditMode = function() {
		return self.widget.mode === 'edit';
	};

	function clearCurrentRow() {
		self.widget.currentRow[self.field.name] = '';
		_callChangeEvent();
		_callAfterDeselectFileEvent();
	}

	this.clearImage = function(file) {

		if (file) {
			clearCurrentRow();

			file.name = ScreenService.i18n("No file selected.");
			return file.name;
		}

		clearCurrentRow();

		self.field.noFileSelectedText = ScreenService.i18n("No file selected.");
		return self.field.noFileSelectedText;
	};

	function _callChangeEvent() {
		if (self.field.change) {
			self.field.change();
		}
	}

	function _callAfterDeselectFileEvent() {
		if (self.field.afterDeselectFile) {
			self.field.afterDeselectFile();
		}
	}
};

ZeedhiController.controller('ImageUploaderController', ImageUploaderController);

// FILE: src/directives/ImageUploaderDirective.js
(function() {
	var ZeedhiDirectives = angular.module('ZeedhiDirectives');
	ZeedhiDirectives.directive('zhImageUploader', zhImageUploader);

	function zhImageUploader(templateManager, ZHPromise) {

		var linkFunction = function(scope, element, attrs) {
			var container = element.parent();
			var field = scope.field;
			var widget = field.widget;
			var filesPromises = [];

			field.fileList = [];
			element.shouldNotPreventDefaultOnClick = true;
			element.parent().find("ul").nonbounce();
			element.on('change', onFileSelected);

			var button = element.siblings('button');
			var input = element;

			button.on('click.file-upload touchstart.file-upload', function(e) {
				input.trigger(e.type);
			});

			function onFileSelected() {
				var files = this.files;

				if (files.length > 0) {
					_ensureCurrentRowPropertyExistence();
					_setCurrentRowAsChanged();
					_putFilesIntoPromises(files);
					_executeAllPromises();
				}

				this.value = "";
			}

			function _ensureCurrentRowPropertyExistence() {
				widget.currentRow = widget.currentRow || {};
				widget.currentRow[field.name] = widget.currentRow[field.name] || [];
			}

			function _setCurrentRowAsChanged() {
				widget.currentRow.changed = true;
			}

			function _putFilesIntoPromises(files) {
				for (var i in files) {
					if (files.hasOwnProperty(i) && files[i] instanceof File) {
						filesPromises.push(_buildFileData(files[i]));
					}
				}
			}

			function _buildFileData(file) {
				var deferred = ZHPromise.defer();
				var fileReader = new FileReader();

				fileReader.onload = function(event) {
					widget.currentRow[field.name] = [_addExtraDataToFile(file)];
					field.fileList = widget.currentRow[field.name];
					deferred.resolve();
				};

				fileReader.readAsDataURL(file);
				return deferred;
			}

			function _addExtraDataToFile(file) {
				// isso causa uma planificacao do objeto do tipo File o que resolve o problema do angular.copy e JSON.stringify
				return $.extend(false, {
					b64File: event.target.result
				}, file);
			}

			function _executeAllPromises() {
				ZHPromise.all(filesPromises)
					.then(function() {
						_callAfterSelectFileEvent();
						field.validate(field.value());
						templateManager.updateTemplate();
					});
			}

			function _callAfterSelectFileEvent() {
				if (field.afterSelectFile) {
					field.afterSelectFile();
				}
			}
		};

		return {
			restrict: 'A',
			link: linkFunction
		};
	}
})();

// FILE: src/registers/ImageUploaderRegister.js
function ImageUploaderRegister(DecodeValueFieldService, decodeTemplateFieldService){
    DecodeValueFieldService.registerType("zh-image-uploader#field/image-uploader.html", "image");

    decodeTemplateFieldService.registerViewTemplate(
            "zh-image-uploader#field/image-uploader.html",
            "zh-image-uploader#field/image-view.html"
        );
}

Configuration(function(ContextRegister){
    ContextRegister.register('ImageUploaderRegister', ImageUploaderRegister);
});